@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -35,7 +35,7 @@ def main():
     opt.clf.choices=['knn', 'lin_nu_svmc', 'rbf_nu_svmc']
     opt.clf.default='lin_nu_svmc'
 
-    parser.add_options([opt.clf, opt.radius, opt.zscore])
+    parser.add_options([opt.clf, opt.zscore])
 
     (options, files) = parser.parse_args()
 
