@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -9,13 +9,27 @@
 """Base classes for Anatomy atlases support
 
 TODOs:
+======
+
  * major optimization. Now code is sloppy and slow -- plenty of checks etc
+
+Module Organization
+===================
+mvpa.atlases.base module contains support for various atlases
+
+.. packagetree::
+   :style: UML
+
+:group Base: BaseAtlas XMLBasedAtlas Label Level LabelsLevel
+:group Talairach: PyMVPAAtlas LabelsAtlas ReferencesAtlas
+:group Exceptions: XMLAtlasException
+
 """
 
 from mvpa.base import externals
 
-externals.exists('lxml', raiseException=True)
-from lxml import etree, objectify
+if externals.exists('lxml', raiseException=True):
+    from lxml import etree, objectify
 
 import os, re
 import numpy as N
@@ -24,8 +38,8 @@ from numpy.linalg import norm
 from mvpa.atlases.transformation import SpaceTransformation, Linear
 from mvpa.misc.support import reuseAbsolutePath
 
-externals.exists('nifti', raiseException=True)
-from nifti import NiftiImage
+if externals.exists('nifti', raiseException=True):
+    from nifti import NiftiImage
 
 from mvpa.base import warning
 if __debug__:
@@ -749,7 +763,7 @@ class ReferencesAtlas(PyMVPAAtlas):
             raise ValueError("Distance should not be negative. "
                              " Thus '%f' is not a legal value" % distance)
         if __debug__:
-			debug('ATL__',
+            debug('ATL__',
                   "Setting maximal distance for queries to be %d" % distance)
         self.__distance = distance
 
