@@ -1,5 +1,6 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+"""This module provides a k-Nearest-Neighbour classifier."""
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -24,13 +25,28 @@ if __debug__:
 
 
 class kNN(Classifier):
-    """k-nearest-neighbour classifier.
+    """
+    k-Nearest-Neighbour classifier.
+
+    This is a simple classifier that bases its decision on the distances
+    between the training dataset samples and the test sample(s). Distances
+    are computed using a customizable distance function. A certain number
+    (`k`)of nearest neighbors is selected based on the smallest distances
+    and the labels of this neighboring samples are fed into a voting
+    function to determine the labels of the test sample.
+
+    Training a kNN classifier is extremely quick, as no actuall training
+    is performed as the training dataset is simply stored in the
+    classifier. All computations are done during classifier prediction.
+
+    .. note::
+      If enabled, kNN stores the votes per class in the 'values' state after
+      calling predict().
 
-    If enabled, it stores the votes per class in the 'values' state after
-    calling predict().
     """
 
-    _clf_internals = [ 'knn', 'non-linear', 'binary', 'multiclass', 'notrain2predict' ]
+    _clf_internals = ['knn', 'non-linear', 'binary', 'multiclass',
+                      'notrain2predict' ]
 
     def __init__(self, k=2, dfx=squared_euclidean_distance,
                  voting='weighted', **kwargs):
@@ -59,14 +75,11 @@ class kNN(Classifier):
         self.__data = None
 
 
-    __doc__ = enhancedDocString('kNN', locals(), Classifier)
-
-
     def __repr__(self):
         """Representation of the object
         """
-        return "kNN(k=%d, enable_states=%s)" % \
-            (self.__k, str(self.states.enabled))
+        return "kNN(k=%d, dfx=%s enable_states=%s)" % \
+            (self.__k, self.__dfx, str(self.states.enabled))
 
 
     def __str__(self):
