@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   Copyright (c) 2008 Emanuele Olivetti <emanuele@relativita.com>
@@ -16,8 +16,8 @@ import numpy as N
 from mvpa.base import externals
 from mvpa.misc.exceptions import InvalidHyperparameterError
 
-externals.exists("scipy", raiseException=True)
-import scipy.linalg as SL
+if externals.exists("scipy", raiseException=True):
+    import scipy.linalg as SL
 
 # no sense to import this module if openopt is not available
 if externals.exists("openopt", raiseException=True):
@@ -266,152 +266,3 @@ class ModelSelector(object):
         return self.log_marginal_likelihood_best
 
     pass
-
-
-
-if __name__ == "__main__":
-
-    import gpr
-    import kernel
-    from mvpa.misc import data_generators
-    from mvpa.base import externals
-    N.random.seed(1)
-
-    if externals.exists("pylab", force=True):
-        import pylab
-    pylab.close("all")
-    pylab.ion()
-
-    from mvpa.datasets import Dataset
-    from mvpa.misc import data_generators
-
-    print "GPR:",
-
-    train_size = 40
-    test_size = 100
-    F = 1
-
-    dataset = data_generators.sinModulated(train_size, F)
-    # print label_train
-
-    dataset_test = data_generators.sinModulated(test_size, F, flat=True)
-    data_test = dataset_test.samples
-    label_test = dataset_test.labels
-    # print label_test
-
-    regression = True
-    logml = True
-
-    k = kernel.KernelSquaredExponential()
-    g = gpr.GPR(k,regression=regression)
-    g.states.enable("log_marginal_likelihood")
-    # g.train_fv = dataset.samples
-    # g.train_labels = dataset.labels
-
-    print "GPR hyperparameters' search through maximization of marginal likelihood on train data."
-    print
-    ms = ModelSelector(g,dataset)
-
-    sigma_noise_initial = 1.0e0 # 0.154142346606
-    sigma_f_initial = 1.0e0 # 0.687554871058
-    length_scale_initial = 1.0e0 # 0.263620251025
-
-    problem =  ms.max_log_marginal_likelihood(hyp_initial_guess=[sigma_noise_initial,sigma_f_initial,length_scale_initial], optimization_algorithm="ralg", ftol=1.0e-8,fixedHypers=N.array([0,0,0],dtype=bool))
-    # problem =  ms.max_log_marginal_likelihood(hyp_initial_guess=[1.0,1.0], optimization_algorithm="ralg", ftol=1.0e-3)
-
-    lml = ms.solve()
-    # print ms.hyperparameters_best
-    sigma_noise_best, sigma_f_best, length_scale_best = ms.hyperparameters_best
-    print
-    print "Best sigma_noise:",sigma_noise_best
-    print "Best sigma_f:",sigma_f_best
-    print "Best length_scale:",length_scale_best
-    print "Best log_marginal_likelihood:",lml
-
-    # Best sigma_noise: 0.154142346606
-    # Best sigma_f: 0.687554871058
-    # Best length_scale: 0.263620251025
-    # Best log_marginal_likelihood: -3.54790161194
-
-    gpr.compute_prediction(sigma_noise_best,sigma_f_best,length_scale_best,regression,dataset,data_test,label_test,F)
-
-
-    print
-    print "GPR ARD on dataset from Williams and Rasmussen 1996:"
-    dataset =  data_generators.wr1996()
-    # dataset.samples = N.hstack([dataset.samples]*10) # enlarge dataset's dimensionality, for testing high dimensions
-
-    # Uncomment the kernel you like:
-
-    # Squared Exponential kernel:
-    k = kernel.KernelSquaredExponential()
-    sigma_noise_initial = 1.0e-0
-    sigma_f_initial = 1.0e-0
-    length_scale_initial = N.ones(dataset.samples.shape[1])*1.0e-0
-    hyp_initial_guess = N.hstack([sigma_noise_initial,sigma_f_initial,length_scale_initial])
-    # fixedHypers = N.array([0,0,0,0,0,0,0,0],dtype=bool)
-    fixedHypers = N.array([0]*(hyp_initial_guess.size),dtype=bool)
-
-    # k = kernel.KernelLinear()
-    # sigma_noise_initial = 1.0e-0
-    # sigma_0_initial = 1.0e-0
-    # Sigma_p_initial = N.ones(dataset.samples.shape[1])*1.0e-3
-    # hyp_initial_guess = N.hstack([sigma_noise_initial,sigma_0_initial,Sigma_p_initial])
-    # # fixedHypers = N.array([0,0,0,0,0,0,0,0],dtype=bool)
-    # fixedHypers = N.array([0]*hyp_initial_guess.size,dtype=bool)
-
-    # k = kernel.KernelConstant()
-    # sigma_noise_initial = 1.0e-0
-    # sigma_0_initial = 1.0e-0
-    # hyp_initial_guess = N.array([sigma_noise_initial, sigma_0_initial])
-    # # fixedHypers = N.array([0,0],dtype=bool)
-    # fixedHypers = N.array([0]*hyp_initial_guess.size,dtype=bool)
-
-    # # Exponential kernel:
-    # k = kernel.KernelExponential()
-    # sigma_noise_initial = 1.0e0
-    # sigma_f_initial = 1.0e0
-    # length_scale_initial = N.ones(dataset.samples.shape[1])*1.0e0
-    # # length_scale_initial = 1.0
-    # hyp_initial_guess = N.hstack([sigma_noise_initial,sigma_f_initial,length_scale_initial])
-    # print "hyp_initial_guess:",hyp_initial_guess
-    # # fixedHypers = N.array([0,0,0,0,0,0,0,0],dtype=bool)
-    # fixedHypers = N.array([0]*(hyp_initial_guess.size),dtype=bool)
-    # # expected results (compute with use_gradient=False):
-    # # objFunValue: 161.97952 (feasible, max constraint =  0)
-    # # [  6.72069299e-04   3.16515151e-01   3.11122154e+00   1.54833211e+00
-    # #    1.94703461e+00   3.11122835e+00   4.37916189e+01   2.65398676e+01]
-
-
-    # # Matern_3_2 kernel:
-    # k = kernel.KernelMatern_3_2()
-    # sigma_noise_initial = 1.0e0
-    # sigma_f_initial = 1.0e0
-    # length_scale_initial = N.ones(dataset.samples.shape[1])*1.0e0
-    # # length_scale_initial = 1.0
-    # hyp_initial_guess = N.hstack([sigma_noise_initial,sigma_f_initial,length_scale_initial])
-    # print "hyp_initial_guess:",hyp_initial_guess
-    # # fixedHypers = N.array([0,0,0,0,0,0,0,0],dtype=bool)
-    # fixedHypers = N.array([0]*(hyp_initial_guess.size),dtype=bool)
-
-
-    # # Rational Quadratic kernel:
-    # k = kernel.KernelRationalQuadratic(alpha=0.5)
-    # sigma_noise_initial = 1.0e0
-    # sigma_f_initial = 1.0e0
-    # length_scale_initial = N.ones(dataset.samples.shape[1])*1.0e0
-    # # length_scale_initial = 1.0
-    # hyp_initial_guess = N.hstack([sigma_noise_initial,sigma_f_initial,length_scale_initial])
-    # print "hyp_initial_guess:",hyp_initial_guess
-    # # fixedHypers = N.array([0,0,0,0,0,0,0,0],dtype=bool)
-    # fixedHypers = N.array([0]*(hyp_initial_guess.size),dtype=bool)
-
-
-    g = gpr.GPR(k,regression=regression)
-    g.states.enable("log_marginal_likelihood")
-    ms = ModelSelector(g,dataset)
-
-    # Note that some kernels does not have gradient yet!
-    problem =  ms.max_log_marginal_likelihood(hyp_initial_guess=hyp_initial_guess, optimization_algorithm="ralg", ftol=1.0e-5,fixedHypers=fixedHypers,use_gradient=True, logscale=True)
-    lml = ms.solve()
-    print ms.hyperparameters_best
