@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -25,21 +25,22 @@ import numpy as N
 
 
 # Rely on SG
-import shogun.Features
-import shogun.Classifier
-import shogun.Regression
-import shogun.Kernel
-import shogun.Library
+from mvpa.base import externals
+if externals.exists('shogun', raiseException=True):
+    import shogun.Features
+    import shogun.Classifier
+    import shogun.Regression
+    import shogun.Kernel
+    import shogun.Library
 
 import operator
 
 from mvpa.misc.param import Parameter
 from mvpa.base import warning
 
-from mvpa.clfs.base import MulticlassClassifier
+from mvpa.clfs.meta import MulticlassClassifier
 from mvpa.clfs._svmbase import _SVM
 from mvpa.misc.state import StateVariable
-from mvpa.clfs.base import Classifier, MulticlassClassifier
 from mvpa.measures.base import Sensitivity
 from mvpa.base import externals
 
@@ -105,14 +106,20 @@ class SVM(_SVM):
 
     num_threads = Parameter(1,
                             min=1,
-                            descr='Number of threads to utilize')
+                            doc='Number of threads to utilize')
 
     # NOTE: gamma is width in SG notation for RBF(Gaussian)
-    _KERNELS = { "linear": (shogun.Kernel.LinearKernel,   ('scale',), LinearSVMWeights),
-                 "rbf" :   (shogun.Kernel.GaussianKernel, ('gamma',), None),
-                 "rbfshift" : (shogun.Kernel.GaussianShiftKernel, ('gamma', 'max_shift', 'shift_step'), None),
-                 "sigmoid" : (shogun.Kernel.SigmoidKernel, ('cache_size', 'gamma', 'coef0'), None),
-                }
+    _KERNELS = {}
+    if externals.exists('shogun', raiseException=True):
+        _KERNELS = { "linear": (shogun.Kernel.LinearKernel,
+                               ('scale',), LinearSVMWeights),
+                     "rbf" :   (shogun.Kernel.GaussianKernel,
+                               ('gamma',), None),
+                     "rbfshift": (shogun.Kernel.GaussianShiftKernel,
+                                 ('gamma', 'max_shift', 'shift_step'), None),
+                     "sigmoid": (shogun.Kernel.SigmoidKernel,
+                                ('cache_size', 'gamma', 'coef0'), None),
+                    }
 
     _KNOWN_PARAMS = [ 'epsilon' ]
     _KNOWN_KERNEL_PARAMS = [ ]
@@ -148,33 +155,45 @@ class SVM(_SVM):
     And yes currently we only implemented parallel training for svmlight,
     however all SVMs can be evaluated in parallel.
     """
-    _KNOWN_IMPLEMENTATIONS = {
-        "libsvm" : (shogun.Classifier.LibSVM, ('C',), ('multiclass', 'binary'), ''),
-        "gmnp" : (shogun.Classifier.GMNPSVM, ('C',), ('multiclass', 'binary'), ''),
-        "gpbt" : (shogun.Classifier.GPBTSVM, ('C',), ('binary',), ''),
-        "gnpp" : (shogun.Classifier.GNPPSVM, ('C',), ('binary',), ''),
+    _KNOWN_IMPLEMENTATIONS = {}
+    if externals.exists('shogun', raiseException=True):
+        _KNOWN_IMPLEMENTATIONS = {
+            "libsvm" : (shogun.Classifier.LibSVM, ('C',),
+                       ('multiclass', 'binary'),
+                        "LIBSVM's C-SVM (L2 soft-margin SVM)"),
+            "gmnp" : (shogun.Classifier.GMNPSVM, ('C',),
+                     ('multiclass', 'binary'),
+                      "Generalized Nearest Point Problem SVM"),
+            # XXX should have been GPDT, shogun has it fixed since some version
+            "gpbt" : (shogun.Classifier.GPBTSVM, ('C',), ('binary',),
+                      "Gradient Projection Decomposition Technique for " \
+                      "large-scale SVM problems"),
+            "gnpp" : (shogun.Classifier.GNPPSVM, ('C',), ('binary',),
+                      "Generalized Nearest Point Problem SVM"),
 
-        ## TODO: Needs sparse features...
-        # "svmlin" : (shogun.Classifier.SVMLin, ''),
-        # "liblinear" : (shogun.Classifier.LibLinear, ''),
-        # "subgradient" : (shogun.Classifier.SubGradientSVM, ''),
-        ## good 2-class linear SVMs
-        # "ocas" : (shogun.Classifier.SVMOcas, ''),
-        # "sgd" : ( shogun.Classifier.SVMSGD, ''),
+            ## TODO: Needs sparse features...
+            # "svmlin" : (shogun.Classifier.SVMLin, ''),
+            # "liblinear" : (shogun.Classifier.LibLinear, ''),
+            # "subgradient" : (shogun.Classifier.SubGradientSVM, ''),
+            ## good 2-class linear SVMs
+            # "ocas" : (shogun.Classifier.SVMOcas, ''),
+            # "sgd" : ( shogun.Classifier.SVMSGD, ''),
 
-        # regressions
-        "libsvr": (shogun.Regression.LibSVR, ('C', 'tube_epsilon',), ('regression',), ''),
-        "krr": (shogun.Regression.KRR, ('tau',), ('regression',), ''),
-        }
+            # regressions
+            "libsvr": (shogun.Regression.LibSVR, ('C', 'tube_epsilon',),
+                      ('regression',),
+                       "LIBSVM's epsilon-SVR"),
+            "krr": (shogun.Regression.KRR, ('tau',), ('regression',),
+                    "Kernel Ridge Regression"),
+            }
 
 
     def __init__(self,
                  kernel_type='linear',
                  **kwargs):
-        """This is the base class of all classifier that utilize so
-        far just SVM classifiers provided by shogun.
+        """Interface class to Shogun's classifiers and regressions.
 
-        TODO Documentation if this all works ;-)
+        Default implementation is 'libsvm'.
         """
 
         svm_impl = kwargs.get('svm_impl', 'libsvm').lower()
@@ -495,10 +514,10 @@ class SVM(_SVM):
 
         if retrainable:
             # we must assign it only if it is retrainable
-            self.states.repredicted = not changed_testdata
+            self.states.repredicted = repredicted = not changed_testdata
             if __debug__:
                 debug("SG__", "Re-assigning learing kernel. Repredicted is %s"
-                      % self.states.repredicted)
+                      % repredicted)
             # return back original kernel
             self.__svm.set_kernel(self.__kernel)
 
