@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -10,7 +10,7 @@
 
 __docformat__ = 'restructuredtext'
 
-import mvpa.misc.copy as copy
+import mvpa.support.copy as copy
 
 import numpy as N
 
@@ -24,7 +24,7 @@ from mvpa.misc.errorfx import meanPowerFx, rootMeanPowerFx, RMSErrorFx, \
      CorrErrorFx, CorrErrorPFx, RelativeRMSErrorFx, MeanMismatchErrorFx, \
      AUCErrorFx
 from mvpa.base import warning
-from mvpa.misc.state import StateVariable, Stateful
+from mvpa.misc.state import StateVariable, ClassWithCollections
 from mvpa.base.dochelpers import enhancedDocString, table2string
 from mvpa.clfs.stats import autoNullDist
 
@@ -254,6 +254,7 @@ class ROCCurve(object):
         # take sets which have values in the shape we can handle
         def _checkValues(set_):
             """Check if values are 'acceptable'"""
+            if len(set_)<3: return False
             x = set_[2]
             # TODO: OPT: need optimization
             if (x is None) or len(x) == 0: return False          # undefined
@@ -525,8 +526,8 @@ class ConfusionMatrix(SummaryStatistics):
 
         # reverse mapping from label into index in the list of labels
         rev_map = dict([ (x[1], x[0]) for x in enumerate(labels)])
-        for iset, (targets, predictions, values) in enumerate(self.sets):
-            for t,p in zip(targets, predictions):
+        for iset, set_ in enumerate(self.sets):
+            for t,p in zip(*set_[:2]):
                 mat_all[iset, rev_map[p], rev_map[t]] += 1
 
 
@@ -773,6 +774,7 @@ class ConfusionMatrix(SummaryStatistics):
         if labels_map_rev is not None:
             labels_rev = [','.join([str(x) for x in labels_map_rev[l]])
                                    for l in labels]
+            labels_map_full = dict(zip(labels_rev, labels))
 
         if labels_order is not None:
             labels_order_filtered = filter(lambda x:x is not None, labels_order)
@@ -782,13 +784,13 @@ class ConfusionMatrix(SummaryStatistics):
                 # We were provided numerical (most probably) set
                 labels_plot = labels_order
             elif len(labels_rev) \
-                     and Set(labels_map.keys()) == labels_order_filtered_set:
+                     and Set(labels_rev) == labels_order_filtered_set:
                 # not clear if right whenever there were multiple labels
                 # mapped into the same
                 labels_plot = []
                 for l in labels_order:
                     v = None
-                    if l is not None: v = labels_map[l]
+                    if l is not None: v = labels_map_full[l]
                     labels_plot += [v]
             else:
                 raise ValueError, \
@@ -815,12 +817,13 @@ class ConfusionMatrix(SummaryStatistics):
         ticks = []
         tick_labels = []
         # populate in a silly way
-
+        reordered_indexes = [labels_indexes[i] for i in labels_plot
+                             if i is not None]
         for i, l in enumerate(labels_plot):
             if l is not None:
                 j = labels_indexes[l]
-                confusionmatrix[i, non_empty] = matrix[j, :]
-                confusionmatrix[non_empty, i] = matrix[:, j]
+                confusionmatrix[i, non_empty] = matrix[j, reordered_indexes]
+                confusionmatrix[non_empty, i] = matrix[reordered_indexes, j]
                 ticks += [i + 0.5]
                 if labels_map_rev is not None:
                     tick_labels += ['/'.join(labels_map_rev[l])]
@@ -841,7 +844,7 @@ class ConfusionMatrix(SummaryStatistics):
 
         # some customization depending on the origin
         xticks_position, yticks, ybottom = {
-            'upper': ('top', ticks[::-1], 0.1),
+            'upper': ('top', [Nlabels-x for x in ticks], 0.1),
             'lower': ('bottom', ticks, 0.2)
             }[origin]
 
@@ -849,13 +852,14 @@ class ConfusionMatrix(SummaryStatistics):
         # Plot
         axi = fig.add_axes([0.15, ybottom, 0.7, 0.7])
         im = axi.imshow(confusionmatrix, interpolation="nearest", origin=origin,
-                        aspect='equal', **kwargs)
+                        aspect='equal', extent=(0, Nlabels, 0, Nlabels),
+                        **kwargs)
 
         # plot numbers
         if numbers:
             numbers_kwargs_ = {'fontsize': 10,
-                            'horizontalalignment': 'center',
-                            'verticalalignment': 'center'}
+                               'horizontalalignment': 'center',
+                               'verticalalignment': 'center'}
             maxv = float(N.max(confusionmatrix))
             colors = [im.to_rgba(0), im.to_rgba(maxv)]
             for i,j in zip(*N.logical_not(mask).nonzero()):
@@ -872,7 +876,7 @@ class ConfusionMatrix(SummaryStatistics):
                 P.text(i+0.5, y+0.5, '%d' % v, alpha=alpha, **numbers_kwargs_)
 
         maxv = N.max(confusionmatrix)
-        boundaries = N.linspace(0, maxv, N.min(maxv, 10), True)
+        boundaries = N.linspace(0, maxv, N.min((maxv, 10)), True)
 
         # Label axes
         P.xlabel("targets")
@@ -893,6 +897,8 @@ class ConfusionMatrix(SummaryStatistics):
         if P.matplotlib.get_backend() == 'TkAgg':
             P.ion()
         P.draw()
+        # Store it primarily for testing
+        self._plotted_confusionmatrix = confusionmatrix
         return fig, im, cb
 
 
@@ -996,9 +1002,92 @@ class RegressionStatistics(SummaryStatistics):
             stats[funcname+'_max'] = N.max(stats[funcname_all])
             stats[funcname+'_min'] = N.min(stats[funcname_all])
 
+        # create ``summary`` statistics, since some per-set statistics
+        # might be uncomputable if a set contains just a single number
+        # (like in the case of correlation coefficient)
+        targets, predictions = [], []
+        for i, (targets_, predictions_, values_) in enumerate(sets):
+            targets += list(targets_)
+            predictions += list(predictions_)
+
+        for funcname, func in funcs.iteritems():
+            funcname_all = 'Summary ' + funcname
+            stats[funcname_all] = func(predictions, targets)
+
         self._stats.update(stats)
 
 
+    def plot(self,
+             plot=True, plot_stats=True,
+             splot=True
+             #labels=None, numbers=False, origin='upper',
+             #numbers_alpha=None, xlabels_vertical=True,
+             #numbers_kwargs={},
+             #**kwargs
+             ):
+        """Provide presentation of regression performance in image
+
+        :Parameters:
+          plot : bool
+            Plot regular plot of values (targets/predictions)
+          plot_stats : bool
+            Print basic statistics in the title
+          splot : bool
+            Plot scatter plot
+
+        :Returns:
+           (fig, im, cb) -- figure, imshow, colorbar
+        """
+        externals.exists("pylab", raiseException=True)
+        import pylab as P
+
+        self.compute()
+        # total number of plots
+        nplots = plot + splot
+
+        # turn off automatic update if interactive
+        if P.matplotlib.get_backend() == 'TkAgg':
+            P.ioff()
+
+        fig = P.gcf()
+        P.clf()
+        sps = []                        # subplots
+
+        nplot = 0
+        if plot:
+            nplot += 1
+            sps.append(P.subplot(nplots, 1, nplot))
+            xstart = 0
+            lines = []
+            for s in self.sets:
+                nsamples = len(s[0])
+                xend = xstart+nsamples
+                xs = xrange(xstart, xend)
+                lines += [P.plot(xs, s[0], 'b')]
+                lines += [P.plot(xs, s[1], 'r')]
+                # vertical line
+                P.plot([xend, xend], [N.min(s[0]), N.max(s[0])], 'k--')
+                xstart = xend
+            if len(lines)>1:
+                P.legend(lines[:2], ('Target', 'Prediction'))
+            if plot_stats:
+                P.title(self.asstring(short='very'))
+
+        if splot:
+            nplot += 1
+            sps.append(P.subplot(nplots, 1, nplot))
+            for s in self.sets:
+                P.plot(s[0], s[1], 'o',
+                       markeredgewidth=0.2,
+                       markersize=2)
+            P.gca().set_aspect('equal')
+
+        if P.matplotlib.get_backend() == 'TkAgg':
+            P.ion()
+        P.draw()
+
+        return fig, sps
+
     def asstring(self, short=False, header=True,  summary=True,
                  description=False):
         """'Pretty print' the statistics"""
@@ -1012,14 +1101,16 @@ class RegressionStatistics(SummaryStatistics):
 
         if short:
             if short == 'very':
-                return "%(# of sets)d sets CCe:%(CCe).2f CCp:%(CCp).2g" \
+                # " RMSE/RMP_t:%(RMSE/RMP_t).2f" \
+                return "%(# of sets)d sets CCe=%(CCe).2f p=%(CCp).2g" \
                        " RMSE:%(RMSE).2f" \
-                       " RMSE/RMP_t:%(RMSE/RMP_t).2f" \
+                       " Summary: " \
+                       "CCe=%(Summary CCe).2f p=%(Summary CCp).2g" \
                        % stats
             else:
-                return "%(# of sets)d sets CCe:%(CCe).2f+-%(CCe_std).3f" \
-                       " RMSE:%(RMSE).2f+-%(RMSE_std).3f" \
-                       " RMSE/RMP_t:%(RMSE/RMP_t).2f+-%(RMSE/RMP_t_std).3f" \
+                return "%(# of sets)d sets CCe=%(CCe).2f+-%(CCe_std).3f" \
+                       " RMSE=%(RMSE).2f+-%(RMSE_std).3f" \
+                       " RMSE/RMP_t=%(RMSE/RMP_t).2f+-%(RMSE/RMP_t_std).3f" \
                        % stats
 
         stats_data = ['RMP_t', 'STD_t', 'RMP_p', 'STD_p']
@@ -1048,6 +1139,10 @@ class RegressionStatistics(SummaryStatistics):
         print_stats(printed, stats_data)
         printed.append(["Results:  "])
         print_stats(printed, stats_)
+        printed.append(["Summary:  "])
+        printed.append(["CCe", _p2(stats['Summary CCe']), "", "p=", '%g' % stats['Summary CCp']])
+        printed.append(["RMSE", _p2(stats['Summary RMSE'])])
+        printed.append(["RMSE/RMP_t", _p2(stats['Summary RMSE/RMP_t'])])
 
         if summary:
             for stat in stats_summary:
@@ -1076,7 +1171,7 @@ class RegressionStatistics(SummaryStatistics):
 
 
 
-class ClassifierError(Stateful):
+class ClassifierError(ClassWithCollections):
     """Compute (or return) some error of a (trained) classifier on a dataset.
     """
 
@@ -1102,7 +1197,7 @@ class ClassifierError(Stateful):
             unless train=False, classifier gets trained if
             trainingdata provided to __call__
         """
-        Stateful.__init__(self, **kwargs)
+        ClassWithCollections.__init__(self, **kwargs)
         self.__clf = clf
 
         self._labels = labels
@@ -1112,7 +1207,7 @@ class ClassifierError(Stateful):
         """Either to train classifier if trainingdata is provided"""
 
 
-    __doc__ = enhancedDocString('ClassifierError', locals(), Stateful)
+    __doc__ = enhancedDocString('ClassifierError', locals(), ClassWithCollections)
 
 
     def __copy__(self):
@@ -1188,6 +1283,9 @@ class ClassifierError(Stateful):
         self._precall(testdataset, trainingdataset)
         error = self._call(testdataset, trainingdataset)
         self._postcall(testdataset, trainingdataset, error)
+        if __debug__:
+            debug('CERR', 'Classifier error on %s: %.2f'
+                  % (testdataset, error))
         return error
 
 
@@ -1245,9 +1343,8 @@ class TransferError(ClassifierError):
     def __copy__(self):
         """Performs deepcopying of the classifier."""
         # TODO -- use ClassifierError.__copy__
-        from mvpa.clfs.base import _deepcopyclf
         out = TransferError.__new__(TransferError)
-        TransferError.__init__(out, _deepcopyclf(self.clf), self.errorfx, self._labels)
+        TransferError.__init__(out, self.clf.clone(), self.errorfx, self._labels)
 
         return out
 
