@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -12,7 +12,7 @@ __docformat__ = 'restructuredtext'
 
 import operator
 import random
-import mvpa.misc.copy as copy
+import mvpa.support.copy as copy
 import numpy as N
 
 from sets import Set
@@ -30,6 +30,8 @@ if __debug__:
     from mvpa.base import debug, warning
 
     def _validate_indexes_uniq_sorted(seq, fname, item):
+        """Helper function to validate that seq contains unique sorted values
+        """
         if operator.isSequenceType(seq):
             seq_unique = N.unique(seq)
             if len(seq) != len(seq_unique):
@@ -816,7 +818,8 @@ class Dataset(object):
                                      for x in labels_map.items()]) + '\n'
 
         def cl_stats(axis, u, name1, name2):
-            # Compute statistics per label
+            """ Compute statistics per label
+            """
             stats = {'min': N.min(spcl, axis=axis),
                      'max': N.max(spcl, axis=axis),
                      'mean': N.mean(spcl, axis=axis),
@@ -980,6 +983,8 @@ class Dataset(object):
         also cause major headaches! Order would is verified when
         running in non-optimized code (if __debug__)
         """
+        ids = copy.deepcopy(ids)
+        
         if ids is None and groups is None:
             raise ValueError, "No feature selection specified."
 
@@ -1054,7 +1059,7 @@ class Dataset(object):
             Flag whether to train the mapper with this dataset before applying
             it.
 
-        TODO: selectFeatures is pretty much 
+        TODO: selectFeatures is pretty much
               applyMapper(featuresmapper=MaskMapper(...))
         """
 
@@ -1395,7 +1400,7 @@ class Dataset(object):
         if len(args) == 1 and isinstance(args[0], tuple):
             args = args[0]
 
-        args_,args = args,()
+        args_, args = args, ()
         for a in args_:
             if isinstance(a, slice) and \
                    isinstance(a.start, basestring):
@@ -1504,9 +1509,10 @@ class Dataset(object):
 
         sample = []
         # for each available class
+        labels = self.labels
         for i, r in enumerate(self.uniquelabels):
             # get the list of pattern ids for this class
-            sample += random.sample( (self.labels == r).nonzero()[0],
+            sample += random.sample( (labels == r).nonzero()[0],
                                      nperlabel[i] )
 
         return self.selectSamples( sample )
@@ -1563,7 +1569,9 @@ class Dataset(object):
 
 
     def defineFeatureGroups(self, definition):
-        """
+        """Assign `definition` to featuregroups
+
+        XXX Feature-groups was not finished to be useful
         """
         if not len(definition) == self.nfeatures:
             raise ValueError, \
