@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -15,7 +15,7 @@ set and include important features successively.
 
 __docformat__ = 'restructuredtext'
 
-from mvpa.misc.copy import copy
+from mvpa.support.copy import copy
 
 from mvpa.featsel.base import FeatureSelection
 from mvpa.featsel.helpers import NBackHistoryStopCrit, \
