@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -278,6 +278,20 @@ class ProjectionMapper(Mapper):
             self.selectOut(self._selector)
 
 
+    def _demeanData(self, data):
+        """Helper which optionally demeans
+        """
+        if self._demean:
+            # demean the training data
+            data = data - self._mean
+
+            if __debug__ and "MAP_" in debug.active:
+                debug("MAP_",
+                      "%s: Mean of data in input space %s was subtracted" %
+                      (self.__class__.__name__, self._mean))
+        return data
+
+
     def _train(self, dataset):
         """Worker method. Needs to be implemented by subclass.
 
@@ -548,6 +562,15 @@ class CombinedMapper(Mapper):
         raise ValueError, "Invalid outId passed to CombinedMapper.getNeighbor()"
 
 
+    def __repr__(self):
+        s = Mapper.__repr__(self).rstrip(' )')
+        # beautify
+        if not s[-1] == '(':
+            s += ' '
+        s += 'mappers=[%s])' % ', '.join([m.__repr__() for m in self._mappers])
+        return s
+
+
 
 class ChainMapper(Mapper):
     """Meta mapper that embedded a chain of other mappers.
@@ -665,3 +688,12 @@ class ChainMapper(Mapper):
         Returns a list of outIds
         """
         return self._mappers[-1].getNeighbor(outId, *args, **kwargs)
+
+
+    def __repr__(self):
+        s = Mapper.__repr__(self).rstrip(' )')
+        # beautify
+        if not s[-1] == '(':
+            s += ' '
+        s += 'mappers=[%s])' % ', '.join([m.__repr__() for m in self._mappers])
+        return s
