@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -15,7 +15,7 @@ __docformat__ = 'restructuredtext'
 if __debug__:
     from mvpa.base import debug
 
-from mvpa.misc.copy import deepcopy
+from mvpa.support.copy import deepcopy
 
 import numpy as N
 
