@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -12,7 +12,7 @@ disk."""
 __docformat__ = 'restructuredtext'
 
 import numpy as N
-import mvpa.misc.copy as copy
+import mvpa.support.copy as copy
 from mvpa.base.dochelpers import enhancedDocString
 from sets import Set
 from re import sub as re_sub
