@@ -1,5 +1,5 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -11,6 +11,10 @@
 It was devised to provide conditional storage 
 """
 
+# XXX: MH: The use of `index` as variable name confuses me. IMHO `index` refers
+#          to a position in a container (i.e. list access). However, in this
+#          file it is mostly used in the context of a `key` for dictionary
+#          access. Can we refactor that?
 __docformat__ = 'restructuredtext'
 
 import operator, copy
@@ -21,253 +25,28 @@ import numpy as N
 
 from mvpa.misc.vproperty import VProperty
 from mvpa.misc.exceptions import UnknownStateError
-from mvpa.base.dochelpers import enhancedClassDocString
+from mvpa.misc.attributes import CollectableAttribute, StateVariable
+from mvpa.base.dochelpers import enhancedDocString
+
+from mvpa.base import externals
 
 if __debug__:
     from mvpa.base import debug
 
 
+_in_ipython = externals.exists('running ipython env')
+# Separators around definitions, needed for ReST, but bogus for
+# interactive sessions
+_def_sep = ('`', '')[int(_in_ipython)]
+
 _object_getattribute = object.__getattribute__
 _object_setattr = object.__setattr__
 
-##################################################################
-# Various attributes which will be collected into collections
-#
-class CollectableAttribute(object):
-    """Base class for any custom behaving attribute intended to become
-    part of a collection.
-
-    Derived classes will have specific semantics:
-
-    * StateVariable: conditional storage
-    * AttributeWithUnique: easy access to a set of unique values
-      within a container
-    * Parameter: attribute with validity ranges.
-
-      - ClassifierParameter: specialization to become a part of
-        Classifier's params collection
-      - KernelParameter: --//-- to become a part of Kernel Classifier's
-        kernel_params collection
-
-    Those CollectableAttributes are to be groupped into corresponding
-    collections for each class by statecollector metaclass, ie it
-    would be done on a class creation (ie not per each object)
-    """
-
-    def __init__(self, name=None, doc=None):
-        self.__doc__ = doc
-        self.__name = name
-        self._value = None
-        self._isset = False
-        self.reset()
-        if __debug__:
-            debug("COL",
-                  "Initialized new collectable %s " % name + `self`)
-
-
-    # Instead of going for VProperty lets make use of virtual method
-    def _getVirtual(self):
-        return self._get()
-
-
-    def _setVirtual(self, value):
-        return self._set(value)
-
-
-    def _get(self):
-        return self._value
-
-
-    def _set(self, val):
-        if __debug__:
-            # Since this call is quite often, don't convert
-            # values to strings here, rely on passing them
-            # withing msgargs
-            debug("COL",
-                  "Setting %(self)s to %(val)s ",
-                  msgargs={'self':self, 'val':val})
-        self._value = val
-        self._isset = True
-
-
-    @property
-    def isSet(self):
-        return self._isset
-
-
-    def reset(self):
-        """Simply reset the flag"""
-        if __debug__ and self._isset:
-            debug("COL", "Reset %s to being non-modified" % self.name)
-        self._isset = False
-
-
-    # TODO XXX unify all bloody __str__
-    def __str__(self):
-        res = "%s" % (self.name)
-        if self.isSet:
-            res += '*'          # so we have the value already
-        return res
-
-
-    def _getName(self):
-        return self.__name
-
-
-    def _setName(self, name):
-        if name is not None:
-            if isinstance(name, basestring):
-                if name[0] == '_':
-                    raise ValueError, \
-                          "Collectable attribute name must not start " \
-                          "with _. Got %s" % name
-            else:
-                raise ValueError, \
-                      "Collectable attribute name must be a string. " \
-                      "Got %s" % `name`
-        self.__name = name
-
-
-    # XXX should become vproperty?
-    # YYY yoh: not sure... someone has to do performance testing
-    #     to see which is more effective. My wild guess is that
-    #     _[gs]etVirtual would be faster
-    value = property(_getVirtual, _setVirtual)
-    name = property(_getName, _setName)
-
-
-
-# XXX think that may be discard hasunique and just devise top
-#     class DatasetAttribute
-class AttributeWithUnique(CollectableAttribute):
-    """Container which also takes care about recomputing unique values
-
-    XXX may be we could better link original attribute to additional
-    attribute which actually stores the values (and do reverse there
-    as well).
-
-    Pros:
-      * don't need to mess with getattr since it would become just another
-        attribute
-
-    Cons:
-      * might be worse design in terms of comprehension
-      * take care about _set, since we shouldn't allow
-        change it externally
-
-    For now lets do it within a single class and tune up getattr
-    """
-
-    def __init__(self, name=None, hasunique=True, doc="Attribute with unique"):
-        CollectableAttribute.__init__(self, name, doc)
-        self._hasunique = hasunique
-        self._resetUnique()
-        if __debug__:
-            debug("UATTR",
-                  "Initialized new AttributeWithUnique %s " % name + `self`)
-
-
-    def reset(self):
-        super(AttributeWithUnique, self).reset()
-        self._resetUnique()
-
-
-    def _resetUnique(self):
-        self._uniqueValues = None
-
-
-    def _set(self, *args, **kwargs):
-        self._resetUnique()
-        CollectableAttribute._set(self, *args, **kwargs)
-
-
-    def _getUniqueValues(self):
-        if self.value is None:
-            return None
-        if self._uniqueValues is None:
-            # XXX we might better use Set, but yoh recalls that
-            #     N.unique was more efficient. May be we should check
-            #     on the the class and use Set only if we are not
-            #     dealing with ndarray (or lists/tuples)
-            self._uniqueValues = N.unique(N.asanyarray(self.value))
-        return self._uniqueValues
-
-    uniqueValues = property(fget=_getUniqueValues)
-    hasunique = property(fget=lambda self:self._hasunique)
-
-# Hooks for comprehendable semantics and automatic collection generation
-class SampleAttribute(AttributeWithUnique):
-    pass
-
-class FeatureAttribute(AttributeWithUnique):
-    pass
-
-class DatasetAttribute(AttributeWithUnique):
-    pass
-
-
-class StateVariable(CollectableAttribute):
-    """Simple container intended to conditionally store the value
-    """
-
-    def __init__(self, name=None, enabled=True, doc="State variable"):
-        CollectableAttribute.__init__(self, name, doc)
-        self._isenabled = enabled
-        self._defaultenabled = enabled
-        if __debug__:
-            debug("STV",
-                  "Initialized new state variable %s " % name + `self`)
-
-
-    def _get(self):
-        if not self.isSet:
-            raise UnknownStateError("Unknown yet value of %s" % (self.name))
-        return CollectableAttribute._get(self)
-
-
-    def _set(self, val):
-        if self.isEnabled:
-            # XXX may be should have left simple assignment
-            # self._value = val
-            CollectableAttribute._set(self, val)
-        elif __debug__:
-            debug("COL",
-                  "Not setting disabled %(self)s to %(val)s ",
-                  msgargs={'self':self, 'val':val})
-
-
-    def reset(self):
-        """Simply detach the value, and reset the flag"""
-        CollectableAttribute.reset(self)
-        self._value = None
-
-
-    @property
-    def isEnabled(self):
-        return self._isenabled
-
-
-    def enable(self, value=False):
-        if self._isenabled == value:
-            # Do nothing since it is already in proper state
-            return
-        if __debug__:
-            debug("STV", "%s %s" %
-                  ({True: 'Enabling', False: 'Disabling'}[value], str(self)))
-        self._isenabled = value
-
-
-    def __str__(self):
-        res = CollectableAttribute.__str__(self)
-        if self.isEnabled:
-            res += '+'          # it is enabled but no value is assigned yet
-        return res
-
 
 ###################################################################
 # Collections
 #
-# TODO: refactor into attributes.py and collections.py. state.py now has
+# TODO: refactor into collections.py. state.py now has
 #       little in common with the main part of this file
 #
 class Collection(object):
@@ -421,7 +200,7 @@ class Collection(object):
 #        operation = { True: copy.deepcopy,
 #                      False: copy.copy }[deep]
 #
-#        if isinstance(fromstate, Stateful):
+#        if isinstance(fromstate, ClassWithCollections):
 #            fromstate = fromstate.states
 #
 #        self.enabled = fromstate.enabled
@@ -647,7 +426,8 @@ class Collection(object):
         # lets assure consistent litsting order
         items = self._items.items()
         items.sort()
-        return [ "%s: %s" % (str(x[1]), x[1].__doc__) for x in items ]
+        return [ "%s%s%s: %s" % (_def_sep, str(x[1]), _def_sep, x[1].__doc__)
+                 for x in items ]
 
 
     def _getNames(self):
@@ -660,9 +440,9 @@ class Collection(object):
 
 
     def _setOwner(self, owner):
-        if not isinstance(owner, Stateful):
+        if not isinstance(owner, ClassWithCollections):
             raise ValueError, \
-                  "Owner of the StateCollection must be Stateful object"
+                  "Owner of the StateCollection must be ClassWithCollections object"
         if __debug__:
             try:    strowner = str(owner)
             except: strowner = "UNDEF: <%s#%s>" % (owner.__class__, id(owner))
@@ -802,7 +582,7 @@ class StateCollection(Collection):
         :Parameters:
           items : dict
             dictionary of states
-          owner : Stateful
+          owner : ClassWithCollections
             object which owns the collection
           name : basestring
             literal description. Usually just attribute name for the
@@ -858,7 +638,7 @@ class StateCollection(Collection):
         """Copy known here states from `fromstate` object into current object
 
         :Parameters:
-          fromstate : Collection or Stateful
+          fromstate : Collection or ClassWithCollections
             Source states to copy from
           index : None or list of basestring
             If not to copy all set state variables, index provides
@@ -882,7 +662,7 @@ class StateCollection(Collection):
         operation = { True: copy.deepcopy,
                       False: copy.copy }[deep]
 
-        if isinstance(fromstate, Stateful):
+        if isinstance(fromstate, ClassWithCollections):
             fromstate = fromstate.states
 
         #self.enabled = fromstate.enabled
@@ -931,7 +711,7 @@ class StateCollection(Collection):
         `enable _states`. Use `resetEnabledTemporarily` to reset
         to previous state of enabled.
 
-        `other` can be a Stateful object or StateCollection
+        `other` can be a ClassWithCollections object or StateCollection
         """
         if enable_states == None:
             enable_states = []
@@ -939,7 +719,7 @@ class StateCollection(Collection):
             disable_states = []
         self.__storedTemporarily.append(self.enabled)
         other_ = other
-        if isinstance(other, Stateful):
+        if isinstance(other, ClassWithCollections):
             other = other.states
 
         if not other is None:
@@ -1001,8 +781,8 @@ class StateCollection(Collection):
         It might be handy to store set of enabled states and then to restore
         it later on. It can be easily accomplished now::
 
-        >>> from mvpa.misc.state import Stateful, StateVariable
-        >>> class Blah(Stateful):
+        >>> from mvpa.misc.state import ClassWithCollections, StateVariable
+        >>> class Blah(ClassWithCollections):
         ...   bleh = StateVariable(enabled=False, doc='Example')
         ...
         >>> blah = Blah()
@@ -1143,25 +923,11 @@ class AttributesCollector(type):
                 paramscols.append(col)
                 # lets at least sort the parameters for consistent output
                 col_items = collections[col].items
-                params = col_items.keys()
+                params = [(v._instance_index, k) for k,v in col_items.iteritems()]
                 params.sort()
-                for param in params:
-                    parameter = col_items[param]
-                    paramsdoc += "  %s" % param
-                    try:
-                        paramsdoc += " : %s" % parameter.allowedtype
-                    except:
-                        pass
-                    paramsdoc += "\n"
-                    try:
-                        doc = parameter.__doc__
-                        try:
-                            doc += " (Default: %s)" % parameter.default
-                        except:
-                            pass
-                        paramsdoc += '\n'.join(textwrapper.wrap(doc))+'\n'
-                    except Exception, e:
-                        pass
+                paramsdoc += '\n'.join(
+                    [col_items[param].doc(indent='  ')
+                     for index,param in params]) + '\n'
 
         # Parameters collection could be taked hash of to decide if
         # any were changed? XXX may be not needed at all?
@@ -1176,8 +942,9 @@ class AttributesCollector(type):
   disable_states : None or list of basestring
     Names of the state variables which should be disabled
 """
-            statesdoc = "Enabled by default are listed with +\n  * "
+            statesdoc = "  * "
             statesdoc += '\n  * '.join(collections['states'].listing)
+            statesdoc += "\n\n(States enabled by default are listed with `+`)"
             if __debug__:
                 debug("COLR", "Assigning __statesdoc to be %s" % statesdoc)
             setattr(cls, "_statesdoc", statesdoc)
@@ -1188,7 +955,7 @@ class AttributesCollector(type):
             setattr(cls, "_paramsdoc", paramsdoc)
 
         if paramsdoc + statesdoc != "":
-            cls.__doc__ = enhancedClassDocString(cls, *bases)
+            cls.__doc__ = enhancedDocString(cls, *bases)
 
 
 
@@ -1225,7 +992,7 @@ class ClassWithCollections(object):
         self.__params_set = False
 
         # need to check to avoid override of enabled states in the case
-           # of multiple inheritance, like both Statefull and Harvestable
+           # of multiple inheritance, like both ClassWithCollectionsl and Harvestable
         if not s__dict__.has_key('_collections'):
             s__class__ = self.__class__
 
@@ -1252,7 +1019,7 @@ class ClassWithCollections(object):
         if __debug__:
             descr = kwargs.get('descr', None)
             debug("COL", "ClassWithCollections.__new__ was done "
-                  "for %s id %s with descr=%s" \
+                  "for %s#%s with descr=%s" \
                   % (s__class__.__name__, id(self), descr))
 
         return self
@@ -1307,11 +1074,11 @@ class ClassWithCollections(object):
             #              + " Valid parameters are %s" % known_params
         if __debug__:
             debug("COL", "ClassWithCollections.__init__ was done "
-                  "for %s id %s with descr=%s" \
+                  "for %s#%s with descr=%s" \
                   % (self.__class__.__name__, id(self), descr))
 
 
-    #__doc__ = enhancedDocString('Stateful', locals())
+    #__doc__ = enhancedDocString('ClassWithCollections', locals())
 
 
     def __getattribute__(self, index):
@@ -1414,14 +1181,7 @@ class ClassWithCollections(object):
 
 
 
-# No actual separation is needed now between ClassWithCollections
-# and a specific usecase.
-Stateful = ClassWithCollections
-Parametrized = ClassWithCollections
-
-
-
-class Harvestable(Stateful):
+class Harvestable(ClassWithCollections):
     """Classes inherited from this class intend to collect attributes
     within internal processing.
 
@@ -1440,25 +1200,25 @@ class Harvestable(Stateful):
     _KNOWN_COPY_METHODS = [ None, 'copy', 'deepcopy' ]
 
 
-    def __init__(self, attribs=None, copy_attribs='copy', **kwargs):
+    def __init__(self, harvest_attribs=None, copy_attribs='copy', **kwargs):
         """Initialize state of harvestable
 
         :Parameters:
-            attribs : list of basestr or dicts
-                What attributes of call to store and return within
-                harvested state variable. If an item is a dictionary,
-                following keys are used ['name', 'copy']
-            copy_attribs : None or basestr
-                Default copying. If None -- no copying, 'copy'
-                - shallow copying, 'deepcopy' -- deepcopying
+          harvest_attribs : list of basestr or dicts
+            What attributes of call to store and return within
+            harvested state variable. If an item is a dictionary,
+            following keys are used ['name', 'copy']
+          copy_attribs : None or basestr
+            Default copying. If None -- no copying, 'copy'
+            - shallow copying, 'deepcopy' -- deepcopying
 
         """
-        Stateful.__init__(self, **kwargs)
+        ClassWithCollections.__init__(self, **kwargs)
 
-        self.__atribs = attribs
+        self.__atribs = harvest_attribs
         self.__copy_attribs = copy_attribs
 
-        self._setAttribs(attribs)
+        self._setAttribs(harvest_attribs)
 
 
     def _setAttribs(self, attribs):
